import logging
from pathlib import Path
from typing import List

import click

from cg.apps.environ import environ_email
from cg.cli.workflow.mip.options import (
    ARGUMENT_CASE_ID,
    EMAIL_OPTION,
    OPTION_DRY,
    OPTION_PANEL_BED,
    PRIORITY_OPTION,
    START_WITH_PROGRAM,
)
from cg.exc import CgError
from cg.meta.workflow.mip import MipAnalysisAPI

LOG = logging.getLogger(__name__)


@click.command("config-case")
@OPTION_DRY
@OPTION_PANEL_BED
@ARGUMENT_CASE_ID
@click.pass_context
def config_case(context: click.Context, case_id: str, panel_bed: str, dry_run: bool):
    """Generate a config for the case_id"""

    analysis_api: MipAnalysisAPI = context.obj["analysis_api"]
    analysis_api.verify_case_id_in_statusdb(case_id)

    panel_bed: str = analysis_api.resolve_panel_bed(panel_bed=panel_bed)
    try:
        config_data: dict = analysis_api.pedigree_config(case_id=case_id, panel_bed=panel_bed)
    except CgError as error:
        LOG.error(error.message)
        raise click.Abort()
    if dry_run:
        click.echo(config_data)
        return
    analysis_api.write_pedigree_config(data=config_data, case_id=case_id)


@click.command()
@OPTION_DRY
@ARGUMENT_CASE_ID
@click.pass_context
def panel(context: click.Context, case_id: str, dry_run: bool):
    """Write aggregated gene panel file"""

    analysis_api: MipAnalysisAPI = context.obj["analysis_api"]
    analysis_api.verify_case_id_in_statusdb(case_id=case_id)

    bed_lines: List[str] = analysis_api.panel(case_id=case_id)
    if dry_run:
        for bed_line in bed_lines:
            click.echo(bed_line)
        return
    analysis_api.write_panel(case_id, bed_lines)


@click.command()
@PRIORITY_OPTION
@EMAIL_OPTION
@START_WITH_PROGRAM
@ARGUMENT_CASE_ID
@OPTION_DRY
@click.option("--mip-dry-run", is_flag=True, help="Run MIP in dry-run mode")
@click.option(
    "--skip-evaluation",
    is_flag=True,
    help="Skip mip qccollect evaluation",
)
@click.pass_context
def run(
    context: click.Context,
    case_id: str,
    dry_run: bool = False,
    email: str = None,
    mip_dry_run: bool = False,
    priority: str = None,
    skip_evaluation: bool = False,
    start_with: str = None,
):
    """Run the analysis for a case"""

    analysis_api: MipAnalysisAPI = context.obj["analysis_api"]

    analysis_api.verify_case_id_in_statusdb(case_id)
    command_args = dict(
        priority=priority or analysis_api.get_priority_for_case(case_id),
        email=email or environ_email(),
        dryrun=mip_dry_run,
        start_with=start_with,
        skip_evaluation=analysis_api.get_skip_evaluation_flag(
            case_id=case_id, skip_evaluation=skip_evaluation
        ),
    )

    analysis_api.check_analysis_ongoing(case_id=case_id)
    analysis_api.run_analysis(case_id=case_id, dry_run=dry_run, command_args=command_args)

    if dry_run:
        LOG.info("Running in dry-run mode.")
        return

    if mip_dry_run:
        LOG.info("Executed MIP in dry-run mode")
        return

    try:
        analysis_api.add_pending_trailblazer_analysis(case_id=case_id)
        analysis_api.set_statusdb_action(case_id=case_id, action="running")
        LOG.info("%s run started!", analysis_api.pipeline)
    except CgError as e:
        LOG.error(e.message)
        raise click.Abort



from typing import List, Optional

from pydantic import BaseModel
from typing_extensions import Literal

from cg.apps.lims import LimsAPI


class LimsSample(BaseModel):
    id: str
    name: str = None
    customer: str = None
    sex: Literal["male", "female", "unknown"] = None
    father: str = None
    mother: str = None
    status: Literal["affected", "unaffected", "unknown"] = None
    application: str = None
    application_version: str = None
    family: str = None
    panels: List[str] = None
    comment: str = None
    project: str = None
    received: str = None
    source: str = None
    priority: str = None


class MockLimsAPI(LimsAPI):
    """Mock LIMS API to get target bed from lims"""

    def __init__(self, config: dict = None, samples: List[dict] = []):
        self.config = config
        self.sample_vars = {}
        self._samples = samples

    def sample(self, sample_id: str) -> Optional[dict]:
        for sample in self._samples:
            if sample["id"] == sample_id:
                return sample
        return None

    def add_sample(self, internal_id: str):
        self.sample_vars[internal_id] = {}

    def add_capture_kit(self, internal_id: str, capture_kit):
        if not internal_id in self.sample_vars:
            self.add_sample(internal_id)
        self.sample_vars[internal_id]["capture_kit"] = capture_kit

    def capture_kit(self, internal_id: str):
        if internal_id in self.sample_vars:
            return self.sample_vars[internal_id].get("capture_kit")
        return None

    def get_prep_method(self, lims_id: str) -> str:
        return (
            "CG002 - End repair Size selection A-tailing and Adapter ligation (TruSeq PCR-free "
            ""
            "DNA)"
        )

    def get_sequencing_method(self, lims_id: str) -> str:
        return "CG002 - Cluster Generation (HiSeq X)"

    def get_delivery_method(self, lims_id: str) -> str:
        return "CG002 - Delivery"

    def update_sample(
        self, lims_id: str, sex=None, target_reads: int = None, name: str = None, **kwargs
    ):
        pass  # This is completely mocked out



from typing import Optional, Type

from cg.constants import DEFAULT_CAPTURE_KIT, Pipeline
from cg.meta.workflow.mip import MipAnalysisAPI
from cg.store import models
from cg.utils import Process


class MipDNAAnalysisAPI(MipAnalysisAPI):
    def __init__(self, config: dict, pipeline: Pipeline = Pipeline.MIP_DNA):
        super().__init__(config, pipeline)

    @property
    def root(self) -> str:
        return self.config["mip-rd-dna"]["root"]

    @property
    def conda_env(self) -> str:
        return self.config["mip-rd-dna"]["conda_env"]

    @property
    def mip_pipeline(self) -> str:
        return self.config["mip-rd-dna"]["pipeline"]

    @property
    def script(self) -> str:
        return self.config["mip-rd-dna"]["script"]

    @property
    def threshold_reads(self):
        return True

    @property
    def process(self) -> Process:
        if not self._process:
            self._process = Process(
                binary=f"{self.script} {self.mip_pipeline}",
                config=self.config["mip-rd-dna"]["mip_config"],
                environment=self.conda_env,
            )
        return self._process

    def config_sample(self, link_obj: models.FamilySample, panel_bed: Optional[str]) -> dict:
        sample_data = self.get_sample_data(link_obj)
        if sample_data["analysis_type"] == "wgs":
            sample_data["capture_kit"] = panel_bed or DEFAULT_CAPTURE_KIT
        else:
            sample_data["capture_kit"] = panel_bed or self.get_target_bed_from_lims(
                link_obj.family.internal_id
            )
        if link_obj.mother:
            sample_data["mother"] = link_obj.mother.internal_id
        if link_obj.father:
            sample_data["father"] = link_obj.father.internal_id
        return sample_data
